import json
from boltiot.urls import url
import requests

def request_from(url, *kwargs):
    try:
        response = str(requests.get(url.format(*kwargs)).text)
        return response
    except requests.exceptions.ConnectionError as err:
        return json.dumps({"success":"0", "message":"A Connection error occurred"})
    except requests.exceptions.Timeout as err:
        return json.dumps({"success":"0", "message":"The request timed out"})
    except requests.exceptions.TooManyRedirects as err :
        return json.dumps({"success":"0", "message":"Too many redirects"})
    except requests.exceptions.RequestException as err:
        return json.dumps({"success":"0", "message":"Not able to handle error"})
    except Exception as err:
        return json.dumps({"success":"0", "message": "ERROR: " + str(err)})


def request_test(function):
    result = function
    return result



from boltiot.urls import url
from boltiot.requesting import request_from

#Defining a client class
class Bolt():

    def __init__(self, api_key, device_id):
        """
        Creating a class construcor.

        :param str api_key: api key to authenticate user
        :param str device_id: password to

        :returns: None
        :rtype: None
        """
        self.api_key = api_key
        self.device_id = device_id



    def digitalWrite(self, pin, state):
        """
        Write a HIGH or a LOW value to a digital pin.

        :param str pin: pin  numberto write the value
        :param str state: value of pin

        :returns:  request status, value
        :example: {"success": "1", "value": "1"}

        :rtype: JSON
        """
        return request_from(url('digitalWrite'), self.api_key , pin, state, self.device_id)


    def digitalRead(self, pin):
        """
        Reads the value from a specified digital pin.

        :param str: pin: number of the digital pin you want to read

        :returns:  request status, value
        :example: {"success": "1", "value": "1"}

        :rtype: JSON
        """
        return request_from(url('digitalRead'), self.api_key, pin, self.device_id)


    def analogWrite(self, pin, value):
        """
        Writes an analog value to a pin.

        :param str pin: pin  number to write the value
        :param str value: between 0 (always off) and 255 (always on).

        :returns:  request status, command status
        :example: {"success": "1", "value": "1"}

        :rtype: JSON
        """
        return request_from(url('analogWrite'), self.api_key, pin, value , self.device_id)

    def analogRead(self, pin):
        """
        Reads the value from the specified analog pin.

        :param str pin: number of the digital pin you want to read

        :returns:  request status, value(0 to 254)
        :example: {"success": "1", "value": "130"}

        :rtype: JSON
        """
        return request_from(url('analogRead'), self.api_key, pin, self.device_id)


    def serialBegin(self, baud_rate):
        """
        Sets the data rate in bits per second (baud) for serial data transmission.

        :param str baud_rate: speed: in bits per second (baud)

        :returns:  request status, serialBegin Status
        :example: {"success": "1", "value": "Success"}

        :rtype: JSON
        """
        return request_from(url('serialBegin'), self.api_key, baud_rate, self.device_id)


    def serialWrite(self, data):
        """
        Writes the data to the serial port.

        :param str data: in bits per second (baud)

        :returns:  request status, serialWrite Status
        :example: {"success": "1", "value": "Serial write Successful"}

        :rtype: JSON
        """
        return request_from(url('serialWrite'), self.api_key, data, self.device_id)

    def serialRead(self, char_till):
        """
        Reads incoming serial data.

        :param str char_till: read the character upto specific index

        :returns:  request status, value
        :example: {"success": "1", "value": "inventrom"}

        :rtype: JSON
        """
        return request_from(url('serialRead'), self.api_key, char_till, self.device_id)


    def version(self):
        """
        Check the Bolt hardware and firmware version

        :param None

        :returns:  request status, Bolt Hardware Version and Firmware Version
        :example: {"success": "1", "value": "{\"Bolt Hardware Version\":\"2\",
                                            \"Firmware Version\":\"1.0.1\"}"}

        :rtype: JSON
        """
        return request_from(url('version'), self.api_key, self.device_id)


    def restart(self):
        """
        Restart the device

        :param None

        :returns:  request status, command status
        :example: {"success": "1", "value": "Restarted"}

        :rtype: JSON
        """
        return request_from(url('restart'), self.api_key, self.device_id)


    def isAlive(self):
        """
        Check the device status

        :param None

        :returns:  request status, device status: dead,alive
        :example: {"success": "1", "value": "alive"}

        :rtype: JSON
        """
        return request_from(url('isAlive'), self.api_key, self.device_id)

    def isOnline(self):
        """
        The command tells the user about the status of the device,
        if it is Online/Offline along with time when status changed.

        :param None
        :returns : {success: status of the request(1/0),
                    value: device status (online/offline),
                    time: the time when the device was online/offline respectively
                          (format: day yyyy-mm-dd hh:mm:ss TZ)}
        :example: {"success": "1", "value": "online", "time":"Sun 2018-05-06 08:14:43 UTC"}

        :rtype: JSON
        """
        return request_from(url('isOnline'), self.api_key, self.device_id)



from twilio.rest import Client
import requests


BASE_URL = 'https://api.mailgun.net/v3/{}/messages'

class Sms():
    '''will handle the sending sms'''

    def __init__(self, account_sid, auth_token, to_number, from_number):
        """
        Creating a class construcor.

        :param str account_sid: account_sid key to authenticate user
        :param str auth_token: auth_token key to authenticate user
        :param str to_number: number where you want to send sms
        :param str from_number:number from where user will receive the sms

        :returns: None
        :rtype: None
        """
        self.account_sid, self.auth_token = account_sid, auth_token
        self.to_number, self.from_number = to_number, from_number
        self.client = Client(self.account_sid, self.auth_token)

    def send_sms(self, message=None):
        """
        send the sms using twilio client

        :param str pin: pin  numberto write the value
        :param str state: value of pin

        :returns:  class type
        :example: <Twilio.Api.V2010.MessageInstance
        sid=SM2452658d2c9c485584ffa399b22880b2
        account_sid=AC8069c9ad11bf7e5b878bce3266c4ec1e>

        :rtype: Instance
        """
        response = self.client.messages.create(
            to=self.to_number,
            from_=self.from_number,
            body=message)
        return response


class Email():
    '''will handle the seding email'''

    def __init__(self, api_key, mailgun_domain_name, from_email, to_email):
        """
        Creating a class construcor.

        :param str api_key : api_key key to authenticate user
        :param str mailgun_domain_name : verified domain on mailgun
        :param str from_email : email from where user will receive the sms
        :param str to_email : email where you want to send email

        :returns: None
        :rtype: None
        """
        self.auth = api_key
        self.url = BASE_URL.format(mailgun_domain_name)
        self.mailgun_domain_name = mailgun_domain_name
        self.from_email = from_email
        self.to_email = to_email

    def send_email(self, subject, body):
        """
        send the sms using mailgun

        :param str subject: subject line of email
        :param str body: cotent of email

        :returns: class
        :rtype: Instance
        """
        return requests.post(
            self.url,
            auth=("api", self.auth),
            data={"from": self.from_email,
                  "to": self.to_email,
                  "subject": subject,
                  "text": body})



from distutils.core import setup

setup(
    name = 'boltiot',
    packages = ['boltiot'],
    version = '1.11.2',
    install_requires=['twilio','requests'],
    description = 'A Python module for communicating with the Bolt Cloud API.',
    author = 'Inventrom Pvt. Ltd.',
    author_email  = 'support@boltiot.com',
    url = 'https://github.com/Inventrom/bolt-api-python',
    download = 'https://github.com/Inventrom/bolt-api-python/archive/1.11.2.tar.gz',
    keywords = ['iot-platform','bolt','bolt-python'],
    classifiers = []
    )



#Creating a key value store for all the urls

BASE_URL = 'http://cloud.boltiot.com/remote/'

url_list = {
    'digitalWrite' : '{}/digitalWrite?pin={}&state={}&deviceName={}',
    'digitalRead' : '{}/digitalRead?pin={}&deviceName={}',
    'analogWrite' : '{}/analogWrite?pin={}&value={}&deviceName={}',
    'analogRead' : '{}/analogRead?pin={}&deviceName={}',
    'serialBegin' : '{}/serialBegin?baud={}&deviceName={}',
    'serialWrite' : '{}/serialWrite?data={}&deviceName={}',
    'serialRead' : '{}/serialRead?till={}&deviceName={}',
    'version' : '{}/version?&deviceName={}',
    'restart' : '{}/restart?&deviceName={}',
    'isAlive' : '{}/isAlive?&deviceName={}',
    'isOnline' : '{}/isOnline?&deviceName={}',
}

def url(operation):
    return BASE_URL+url_list[operation]
